@@ -89,7 +89,7 @@
 
 #define SASL_VERSION_MAJOR 1
 #define SASL_VERSION_MINOR 5
-#define SASL_VERSION_STEP 15
+#define SASL_VERSION_STEP 16
 
 /* The following ifdef block is the standard way of creating macros
  * which make exporting from a DLL simpler. All files within this DLL
@@ -390,6 +390,7 @@ typedef int sasl_getpath_t(void * context,
  * inputs: 
  *  context     -- verifypath context from the callback record
  *  file        -- full path to file to verify
+ *  type        -- type of file to verify
 
  * returns:
  *  SASL_OK        -- no error (file can safely be used)
@@ -397,10 +398,16 @@ typedef int sasl_getpath_t(void * context,
  *  SASL_FAIL      -- error 
  */
 typedef int sasl_verifyfile_t(void * context,
-                              const char * file);
+                              const char * file, const int type);
 
 #define SASL_CB_VERIFYFILE  (4)
 
+/* these are the types of files libsasl will ask about */
+#define SASL_VRFY_PLUGIN	(1)
+#define SASL_VRFY_CONF		(2)
+#define SASL_VRFY_PASSWD	(3)
+#define SASL_VRFY_OTHER		(4)
+
 /* client/user interaction callbacks:
  */
 /* Simple prompt -- result must persist until next call to getsimple or
