@@ -30,6 +30,9 @@ SOFTWARE.
 #include <saslint.h>
 #include <saslutil.h>
 #include <assert.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
 #ifdef HAVE_KRB
 #include <krb.h>
@@ -41,12 +44,12 @@ SOFTWARE.
 #include <netdb.h>
 #include <netinet/in.h>
 #include <sys/param.h>
+#include <sys/un.h>
 #else
 #include <string.h>
 #endif
 
 #include <sys/types.h>
-#include <sys/un.h>
 #include <ctype.h>
 
 #ifndef SASL_MINIMAL_SERVER
@@ -215,6 +218,7 @@ int _sasl_kerberos_verify_password(sasl_conn_t *conn,
     char cell[REALM_SZ];
     char hostname[MAXHOSTNAMELEN+1];
     char phost[MAXHOSTNAMELEN+1];
+    char oldtkfile[MAXPATHLEN];
     KTEXT_ST authent;
     char instance[INST_SZ];
     AUTH_DAT kdata;
@@ -238,6 +242,8 @@ int _sasl_kerberos_verify_password(sasl_conn_t *conn,
 
     if (krb_get_lrealm(realm, 1)) return SASL_FAIL;
 
+    /* after this point, we need to "goto fini" on failure */
+    strcpy(oldtkfile, tkt_string());
     sprintf(tfname, "/tmp/tkt_%d", getpid());
     krb_set_tkt_string(tfname);
 
@@ -260,9 +266,9 @@ int _sasl_kerberos_verify_password(sasl_conn_t *conn,
     memset(key, 0, sizeof(key));
 
     if (result != 0) {
-	dest_tkt();
 	if (reply) *reply = krb_err_txt[result];
-	return SASL_FAIL;
+	result = SASL_FAIL;
+	goto fini;
     }
 
     /* Check validity of returned ticket */
@@ -273,9 +279,9 @@ int _sasl_kerberos_verify_password(sasl_conn_t *conn,
 
     if (result != 0) {
 	memset(&authent, 0, sizeof(authent));
-	dest_tkt();
 	if (reply) *reply = krb_err_txt[result];
-	return SASL_FAIL;
+	result = SASL_FAIL;
+	goto fini;
     }
     strcpy(instance, "*");
 
@@ -295,7 +301,11 @@ int _sasl_kerberos_verify_password(sasl_conn_t *conn,
     }
     else result = SASL_OK;
 
+ fini:
+    /* destroy any tickets we managed to acquire here */
     dest_tkt();
+    /* restore the old ticket file */
+    krb_set_tkt_string(oldtkfile);
     return result;
 }
 
@@ -554,21 +564,26 @@ int _sasl_sasldb_verify_password(sasl_conn_t *conn,
     sasl_secret_t *construct = NULL;
     char *userid, *realm;
 
+    VL(("Verifying password via sasldb\n"));
+
     if (!userstr || !passwd) {
 	return SASL_BADPARAM;
     }
     if (reply) { *reply = NULL; }
     ret = parseuser(&userid, &realm, user_realm, conn->serverFQDN, userstr);
     if (ret != SASL_OK) {
+	VL(("Error parsing user\n"));
 	return ret;
     }
     ret = _sasl_getcallback(conn, SASL_CB_SERVER_GETSECRET, &getsec, &context);
     if (ret != SASL_OK) {
+	VL(("Error getting getsecret callback\n"));
 	return ret;
     }
 
     ret = getsec(context, "PLAIN", userid, realm, &secret);
     if (ret != SASL_OK) {
+	VL(("Error getting PLAIN secret for user %s in realm %s from db\n",userid, realm));
 	return ret;
     }
 
@@ -581,8 +596,10 @@ int _sasl_sasldb_verify_password(sasl_conn_t *conn,
     }
 
     if (!memcmp(secret->data, construct->data, secret->len)) {
+	VL(("Password verified!\n"));
 	ret = SASL_OK;
     } else {
+	VL(("Passwords do not match"));
 	ret = SASL_BADAUTH;
     }
 
