@@ -38,10 +38,13 @@ SOFTWARE.
 #include <stdio.h>
 /* need to handle the fact that errno has been defined as a function
    in a dll, not an extern int */
-#ifdef errno
-#undef errno
-#endif /* errno */
+# ifdef errno
+#  undef errno
+# endif /* errno */
 #endif /* WIN32 */
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
 static const char build_ident[] = "$Build: libsasl " PACKAGE "-" VERSION " $";
 
@@ -169,6 +172,10 @@ void sasl_done(void)
 
   if (_sasl_client_cleanup_hook)
     _sasl_client_cleanup_hook();
+
+  /* in case of another init/done */
+  _sasl_server_cleanup_hook = NULL;
+  _sasl_client_cleanup_hook = NULL;
 }
 
 /* fills in the base sasl_conn_t info */
@@ -444,18 +451,20 @@ _sasl_global_getopt(void *context,
 
   global_callbacks = (const sasl_global_callbacks_t *) context;
 
-  if (global_callbacks && global_callbacks->callbacks)
-    for (callback = global_callbacks->callbacks;
-	 callback->id != SASL_CB_LIST_END;
-	 callback++)
-      if (callback->id == SASL_CB_GETOPT
-	  && (((sasl_getopt_t *)(callback->proc))(callback->context,
-						  plugin_name,
-						  option,
-						  result,
-						  len)
-	      == SASL_OK))
-	return SASL_OK;
+  if (global_callbacks && global_callbacks->callbacks) {
+      for (callback = global_callbacks->callbacks;
+	   callback->id != SASL_CB_LIST_END;
+	   callback++) {
+	  if (callback->id == SASL_CB_GETOPT
+	      && (((sasl_getopt_t *)(callback->proc))(callback->context,
+						      plugin_name,
+						      option,
+						      result,
+						      len)
+		  == SASL_OK))
+	      return SASL_OK;
+      }
+  }
 
   /* look it up in our configuration file */
   *result = sasl_config_getstring(option, NULL);
@@ -610,7 +619,8 @@ _sasl_getpath(void *context __attribute__((unused)),
 
 static int
 _sasl_verifyfile(void *context __attribute__((unused)),
-		 char *file  __attribute__((unused)))
+		 char *file  __attribute__((unused)),
+		 int type  __attribute__((unused)))
 {
   /* always say ok */
   return SASL_OK;
@@ -627,7 +637,9 @@ _sasl_proxy_policy(void *context __attribute__((unused)),
     *user = NULL;
     if (!auth_identity || !requested_user || 
 	(strcmp(auth_identity, requested_user) != 0)) {
-	*errstr = "Requested identity not authenticated identity";
+	printf("%s vs %s\n",auth_identity, requested_user);
+	if (errstr)
+	    *errstr = "Requested identity not authenticated identity";
 	return SASL_BADAUTH;
     }
 
@@ -658,7 +670,7 @@ _sasl_getcallback(sasl_conn_t * conn,
 	  *pproc = &_sasl_global_getopt;
 	  *pcontext = NULL;
       }
-    return SASL_OK;
+      return SASL_OK;
   }
 
   /* If it's not always provided by the library, see if there's
