@@ -33,6 +33,23 @@ SOFTWARE.
 #include <sasl.h>
 #include "saslint.h"
 
+#if HAVE_DIRENT_H
+# include <dirent.h>
+# define NAMLEN(dirent) strlen((dirent)->d_name)
+#else /* HAVE_DIRENT_H */
+# define dirent direct
+# define NAMLEN(dirent) (dirent)->d_namlen
+# if HAVE_SYS_NDIR_H
+#  include <sys/ndir.h>
+# endif
+# if HAVE_SYS_DIR_H
+#  include <sys/dir.h>
+# endif
+# if HAVE_NDIR_H
+#  include <ndir.h>
+# endif
+#endif /* ! HAVE_DIRENT_H */
+
 #ifndef PATH_MAX
 # ifdef _POSIX_PATH_MAX
 #  define PATH_MAX _POSIX_PATH_MAX
@@ -101,7 +118,7 @@ dlsym(dll_handle h, char *n)
 char *dlerror()
 {
     if (errno != 0) {
-	return sterror(errno);
+	return strerror(errno);
     }
     return "Generic shared library error";
 }
@@ -152,14 +169,18 @@ int _sasl_get_mech_list(const char *entryname,
   /* get the path to the plugins */
   result = ((sasl_getpath_t *)(getpath_cb->proc))(getpath_cb->context,
 						  &path);
+  result = SASL_OK;
+
   if (result != SASL_OK)
     return result;
 
   if (! path) {
     free_path = 0;
     path = PLUGINDIR;
-  } else
+  } else {
     free_path = 1;
+  }
+
   /* xxx sendmail guys -NAME_MAX */
   if (strlen(path)>=PATH_MAX) { /* no you can't buffer overrun */
     if (free_path)
@@ -209,7 +230,7 @@ int _sasl_get_mech_list(const char *entryname,
 	
 	/* Ask the application if we should use this file or not */
 	result = ((sasl_verifyfile_t *)(verifyfile_cb->proc))
-	    (verifyfile_cb->context, tmp);
+	    (verifyfile_cb->context, tmp, SASL_VRFY_PLUGIN);
 	/* returns continue if this file is to be skipped */
 	if (result == SASL_CONTINUE) continue; 
 	
@@ -248,7 +269,7 @@ int _sasl_get_mech_list(const char *entryname,
   } while ((c!='=') && (c!=0));
 
   if (free_path)
-    sasl_FREE(path);
+      sasl_FREE(path); 
 
   return SASL_OK;
 }
