@@ -1,6 +1,6 @@
 /* SASL server API implementation
  * Tim Martin
- * $Id: server.c,v 1.64 1999/12/31 04:39:50 leg Exp $
+ * $Id: server.c,v 1.68 2000/02/24 01:36:22 leg Exp $
  */
 /***********************************************************
         Copyright 1998 by Carnegie Mellon University
@@ -36,15 +36,7 @@ SOFTWARE.
 #include <fcntl.h>
 #include <string.h>
 
-#ifdef HAVE_PAM
-#define DEFAULT_PLAIN_MECHANISM "PAM"
-#else
-# ifdef HAVE_KRB
-#define DEFAULT_PLAIN_MECHANISM "kerberos_v4"
-# else
-#define DEFAULT_PLAIN_MECHANISM "passwd"
-# endif
-#endif
+#define DEFAULT_PLAIN_MECHANISM "sasldb"
 
 #include "sasl.h"
 #include "saslint.h"
@@ -286,12 +278,18 @@ int sasl_setpass(sasl_conn_t *conn,
     void *context;
     const char *plainmech;
 
+    /* check params */
     if (!conn)
 	return SASL_BADPARAM;
     
     if (!mechlist) {
 	return SASL_FAIL;
     }
+    if (passlen == 0)
+    {
+	if (errstr) *errstr = "Password must be at least one character long";
+	return SASL_BADPARAM;
+    }
 
     if ((flags & SASL_SET_CREATE) && (flags & SASL_SET_DISABLE)) {
 	return SASL_BADPARAM;
@@ -351,7 +349,7 @@ int sasl_setpass(sasl_conn_t *conn,
 	    _sasl_log(conn, SASL_LOG_INFO, m->plug->mech_name, 0, 0,
 		      "secret not changed for %s", user);
 	} else {
-	    result = SASL_FAIL;
+	    result = tmpresult;
 	    _sasl_log(conn, SASL_LOG_ERR, m->plug->mech_name, tmpresult,
 #ifndef WIN32
 		      errno,
@@ -548,7 +546,7 @@ static int load_config(const sasl_callback_t *verifyfile_cb)
 
   /* Ask the application if it's safe to use this file */
   result = ((sasl_verifyfile_t *)(verifyfile_cb->proc))(verifyfile_cb->context,
-							config_filename);
+					config_filename, SASL_VRFY_CONF);
 
   /* returns continue if this file is to be skipped */
   
@@ -566,6 +564,24 @@ static int load_config(const sasl_callback_t *verifyfile_cb)
   return result;
 }
 
+/*
+ * Verify that all the callbacks are valid
+ */
+
+static int verify_server_callbacks(const sasl_callback_t *callbacks)
+{
+    if (callbacks == NULL) return SASL_OK;
+
+    while (callbacks->id != SASL_CB_LIST_END)
+    {
+	if (callbacks->proc==NULL) return SASL_FAIL;
+
+	callbacks++;
+    }
+
+    return SASL_OK;
+}
+
 /* initialize server drivers, done once per process
  *  callbacks      -- base callbacks for all server connections
  *  appname        -- name of calling application (for lower level logging)
@@ -584,12 +600,16 @@ int sasl_server_init(const sasl_callback_t *callbacks,
   int ret;
   const sasl_callback_t *vf;
 
-  _sasl_server_cleanup_hook = &server_done;
-  _sasl_server_idle_hook = &server_idle;
+  /* we require the appname to be non-null */
+  if (appname==NULL) return SASL_BADPARAM;
 
   _sasl_server_getsecret_hook = _sasl_db_getsecret;
   _sasl_server_putsecret_hook = _sasl_db_putsecret;
 
+  /* verify that the callbacks look ok */
+  ret = verify_server_callbacks(callbacks);
+  if (ret!=SASL_OK) return ret;
+
   global_callbacks.callbacks = callbacks;
   global_callbacks.appname = appname;
 
@@ -619,6 +639,14 @@ int sasl_server_init(const sasl_callback_t *callbacks,
 			  _sasl_find_verifyfile_callback(callbacks),
 			  &add_plugin);
 
+  
+  if (ret == SASL_OK)
+  {
+      /* cleanup_hook shows if we're active or not. sasl_done() sets it back to null */
+      _sasl_server_cleanup_hook = &server_done;
+      _sasl_server_idle_hook = &server_idle;
+  }
+
   return ret;
 }
 
@@ -1003,6 +1031,9 @@ int sasl_listmech(sasl_conn_t *conn,
   int flag;
   const char *mysep;
 
+  /* if there hasn't been a sasl_sever_init() fail */
+  if (_sasl_server_cleanup_hook==NULL) return SASL_FAIL;
+
   if (! conn || ! result)
     return SASL_FAIL;
 
