@@ -26,13 +26,14 @@ SOFTWARE.
 #include <config.h>
 #include <sasl.h>
 #include <saslplug.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
 #ifdef WIN32
 /* This must be after sasl.h, saslutil.h */
 # include "saslANONYMOUS.h"
-#else /* WIN32 */
-#include <netinet/in.h>
-#endif /* WIN32 */
+#endif
 
 static const char rcsid[] = "$Implementation: Carnegie Mellon SASL " VERSION " $";
 
@@ -84,6 +85,9 @@ server_continue_step (void *conn_context __attribute__((unused)),
       || !oparams)
     return SASL_BADPARAM;
 
+  if (clientinlen < 0)
+      return SASL_BADPARAM;
+
   if (errstr)
       *errstr = NULL;
 
@@ -109,7 +113,7 @@ server_continue_step (void *conn_context __attribute__((unused)),
     sparams->utils->log(sparams->utils->conn,
 			SASL_LOG_INFO,
 			"ANONYMOUS", 0, 0,
-			"login: \"%s\" from [%i.%i.%i.%i]",
+			"login: \"%s\" from [%d.%d.%d.%d]",
 			clientdata,
 			ipnum >> 24 & 0xFF,
 			ipnum >> 16 & 0xFF,
@@ -287,7 +291,7 @@ client_continue_step(void *conn_context __attribute__((unused)),
   userlen = strlen(user);
 
   memset(hostname, 0, sizeof(hostname));
-  gethostname(hostname,sizeof(hostname));
+  gethostname(hostname, sizeof(hostname));
   hostname[sizeof(hostname)-1] = '\0';
 
   
