@@ -1,6 +1,6 @@
 /* CRAM-MD5 SASL plugin
  * Tim Martin 
- * $Id: cram.c,v 1.45 1999/12/23 22:48:19 leg Exp $
+ * $Id: cram.c,v 1.47 2000/02/23 02:50:01 leg Exp $
  */
 /***********************************************************
         Copyright 1998 by Carnegie Mellon University
@@ -356,6 +356,14 @@ static int server_continue_step (void *conn_context,
      */
     VL(("CRAM-MD5 step 1\n"));
 
+    /* we shouldn't have received anything */
+    if (clientinlen!=0)
+    {
+	if (errstr)
+	    *errstr = "CRAM-MD5 does not accpet inital data";
+	return SASL_FAIL;
+    }
+
     /* get time and a random number for the nonce */
     time=gettime(sparams);
     randdigits=randomdigits(sparams);
@@ -562,10 +570,10 @@ static int mechanism_db_filled(char *mech_name, sasl_utils_t *utils)
     return SASL_FAIL;
   }
 
-  /* Request secret */
+  /* Request "magic" secret */
   result = getsecret(getsecret_context, mech_name, "", "", &sec);
-  if (result == SASL_NOUSER) {
-      return SASL_NOUSER;
+  if (result == SASL_NOUSER || result == SASL_FAIL) {
+      return result;
   }
 
   /* check version */
