@@ -40,24 +40,22 @@ SOFTWARE.
    library routines
 */
 
-#ifdef WITH_CMU_RC4
-#define WITH_RC4
-#endif /* WITH_CMU_RC4 */
-
-#ifdef WITH_SSL_RC4
-#define WITH_RC4
-#endif /* WITH_SSL_RC4 */
-
-
-
+/* DES support */
 #ifdef WITH_DES
-#include <des.h>
+# ifdef WITH_SSL_DES
+#  include <openssl/des.h>
+# else /* system DES library */
+#  include <des.h>
+# endif
 #endif /* WITH_DES */
+
+/* RC4 support */
 #ifdef WITH_CMU_RC4
-#include <rc4.h>
-#endif /* WITH_CMU_RC4 */
-#ifdef WITH_SSL_RC4
-#include <openssl/rc4.h>
+# define WITH_RC4
+# include <rc4.h>
+#elif WITH_SSL_RC4
+# define WITH_RC4
+# include <openssl/rc4.h>
 #endif /* WITH_SSL_RC4 */
 
 #ifdef WIN32
@@ -498,8 +496,9 @@ calculate_response(context_t * text,
       /* a NULL realm is equivalent to the empty string */
       realm = (unsigned char *) "";
   }
+
+  if (nonce == NULL) return NULL;
   
-  assert(nonce != NULL);
   assert(cnonce != NULL);
 
   assert(ncvalue != NULL);
@@ -2626,8 +2625,8 @@ static int mechanism_db_filled(char *mech_name, sasl_utils_t *utils)
       }
       free(sec);
   }
-  if (result == SASL_NOUSER) {
-      return SASL_NOUSER;
+  if (result == SASL_NOUSER || result == SASL_FAIL) {
+      return result;
   }
 
   if (tmpversion != DIGEST_MD5_VERSION)
@@ -3260,6 +3259,10 @@ c_continue_step(void *conn_context,
   context_t      *text;
   text = conn_context;
 
+  /* check params */
+  if (serverinlen < 0)
+      return SASL_BADPARAM;
+
   if (text->state == 1) {
     VL(("Digest-MD5 Step 1\n"));
 
@@ -3322,6 +3325,10 @@ c_continue_step(void *conn_context,
 	char *name, *value;
 
 	get_pair(&in, &name, &value);
+
+	/* if parse error */
+	if (name == NULL) return SASL_FAIL;
+
 	VL(("received pair: %s - %s\n", name, value));
 
 	if (strcmp(name, "realm") == 0) {
@@ -3844,6 +3851,12 @@ FreeAllocatedMem:
       char           *name, *value;
       get_pair(&in, &name, &value);
 
+      if (name == NULL)
+      {
+	  VL (("Received garbage\n"));
+	  return SASL_FAIL;
+      }
+
       VL(("received pair: %s - %s\n", name, value));
 
       if (strcmp(name, "rspauth") == 0) {
