@@ -1,6 +1,6 @@
 /* Kerberos4 SASL plugin
  * Tim Martin 
- * $Id: kerberos4.c,v 1.46 1999/12/02 05:21:03 tmartin Exp $
+ * $Id: kerberos4.c,v 1.53 2000/02/24 01:36:22 leg Exp $
  */
 /***********************************************************
         Copyright 1998 by Carnegie Mellon University
@@ -36,6 +36,9 @@ SOFTWARE.
 # include <arpa/inet.h>
 # include <netdb.h>
 #endif /* WIN32 */
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include <fcntl.h>
 #include <sasl.h>
 #include <saslutil.h>
@@ -551,8 +554,7 @@ static int server_continue_step (void *conn_context,
     return SASL_CONTINUE;
   }
 
-  if (text->state==1)
-  {
+  if (text->state == 1) {
     int nchal;
     unsigned char sout[8];  
     AUTH_DAT ad;
@@ -580,26 +582,26 @@ static int server_continue_step (void *conn_context,
     /* we ignore IP addresses in krb4 tickets at CMU to facilitate moving
        from machine to machine */
 
-    /* check ticket */
-    result=krb_rd_req(&ticket, (char *) sparams->service,
-		      text->instance, 0L, &ad, srvtab);
+    addr = NULL;
 #else
     /* get ip number in addr*/
     result = sparams->utils->getprop(sparams->utils->conn,
 				     SASL_IP_REMOTE, (void **)&addr);
     if (result != SASL_OK) {
-	VL(("getprop SASL_IP_REMOTE failed\n"));
+	if (errstr)
+	    *errstr = "couldn't get remote IP address";
 	return SASL_BADAUTH;
     }
-
-    /* check ticket */
-    result=krb_rd_req(&ticket, (char *) sparams->service,
-		      text->instance, addr->sin_addr.s_addr, &ad, srvtab);
 #endif
+    /* check ticket */
+    result = krb_rd_req(&ticket, (char *) sparams->service, text->instance, 
+			addr ? addr->sin_addr.s_addr : 0L, &ad, srvtab);
 
     if (result) { /* if fails mechanism fails */
 	VL(("krb_rd_req failed service=%s instance=%s error code=%i\n",
 	    sparams->service, text->instance,result));
+	if (errstr)
+	    *errstr = krb_err_txt[result];
 	return SASL_BADAUTH;
     }
 
@@ -672,47 +674,59 @@ static int server_continue_step (void *conn_context,
 
     testnum=(in[0]*256*256*256)+(in[1]*256*256)+(in[2]*256)+in[3];
 
-
-    if (testnum!=text->challenge)
-    {
-      return SASL_BADAUTH;
+    if (testnum != text->challenge) {
+	if (errstr)
+	    *errstr = "incorrect response to challenge";
+	return SASL_BADAUTH;
     }
 
-    if (! cando_sec(&sparams->props, in[4] & KRB_SECFLAGS))
-      return SASL_BADPROT;
+    if (! cando_sec(&sparams->props, in[4] & KRB_SECFLAGS)) {
+	if (errstr)
+	    *errstr = "invalid security property specified";
+	return SASL_BADPROT;
+    }
     
     switch (in[4] & KRB_SECFLAGS) {
     case KRB_SECFLAG_NONE:
-      oparams->encode=NULL;
-      oparams->decode=NULL;
-      oparams->mech_ssf=0;
-      break;
+	oparams->encode=NULL;
+	oparams->decode=NULL;
+	oparams->mech_ssf=0;
+	break;
     case KRB_SECFLAG_INTEGRITY:
-      oparams->encode=&integrity_encode;
-      oparams->decode=&integrity_decode;
-      oparams->mech_ssf=KRB_INTEGRITY_BITS;
-      break;
+	oparams->encode=&integrity_encode;
+	oparams->decode=&integrity_decode;
+	oparams->mech_ssf=KRB_INTEGRITY_BITS;
+	break;
     case KRB_SECFLAG_ENCRYPTION:
-      oparams->encode=&privacy_encode;
-      oparams->decode=&privacy_decode;
-      oparams->mech_ssf=KRB_DES_SECURITY_BITS;
-      break;
+	oparams->encode=&privacy_encode;
+	oparams->decode=&privacy_decode;
+	oparams->mech_ssf=KRB_DES_SECURITY_BITS;
+	break;
     default:
       /* not a supported encryption layer */
-      return SASL_BADPROT;
+	if (errstr)
+	    *errstr = "unsupported layer";
+	return SASL_BADPROT;
     }
 
     /* get ip data */
     result = sparams->utils->getprop(sparams->utils->conn,
 				     SASL_IP_LOCAL,
 				     (void **)&(text->ip_local));
-    if (result != SASL_OK) return result;
+    if (result != SASL_OK) {
+	if (errstr)
+	    *errstr =  "couldn't get local IP address";
+	return result;
+    }
 
     result = sparams->utils->getprop(sparams->utils->conn,
 				     SASL_IP_REMOTE,
 				     (void **)&(text->ip_remote));
-
-    if (result!=SASL_OK) return result;
+    if (result != SASL_OK) {
+	if (errstr)
+	    *errstr = "couldn't get remote IP address";
+	return result;
+    }
 
     text->malloc=sparams->utils->malloc;        
     text->free=sparams->utils->free;
@@ -820,7 +834,7 @@ int sasl_server_plug_init(sasl_utils_t *utils,
     strcpy(srvtab, ret);
 
     /* fail if we can't open the srvtab file */
-    if (access(srvtab, R_OK)!=0)
+    if (access(srvtab, R_OK) != 0)
 	return SASL_FAIL;
 
     if (maxversion<KERBEROS_VERSION)
@@ -893,10 +907,13 @@ static int client_continue_step (void *conn_context,
 
     text->challenge=ntohl(text->challenge); 
 
-    if (params->serverFQDN==NULL)
-    {
-      VL(("No serverFQDN set\n"));
-      return SASL_BADAUTH;
+    if (params->serverFQDN == NULL) {
+	VL(("No serverFQDN set\n"));
+	return SASL_BADPARAM;
+    }
+    if (params->service == NULL) {
+	VL(("No service set\n"));
+	return SASL_BADPARAM;
     }
     if (params->service==NULL)
     {
@@ -917,11 +934,11 @@ static int client_continue_step (void *conn_context,
     VL (("instance=%s\n",text->instance));
 
     if ((result=krb_mk_req(&ticket, service, text->instance,
-			   text->realm,text->challenge)))
+			   text->realm,text->challenge))) 
     {
-      VL(("krb_mk_req failed service=%s instance=%s realm=%s krb error=%i\n",
-	     service,text->instance,text->realm,result));
-      return SASL_FAIL;
+	VL(("krb_mk_req failed service=%s instance=%s realm=%s krb error=%d\n",
+	    service,text->instance,text->realm,result));
+	return SASL_FAIL;
     }
     
     *clientout=params->utils->malloc(ticket.length);
@@ -1156,11 +1173,19 @@ static int client_continue_step (void *conn_context,
     /* nothing more to do; should be authenticated */
     result = params->utils->getprop(params->utils->conn,
                           SASL_IP_LOCAL, (void **)&(text->ip_local));
-    if (result != SASL_OK) return result;
+    if (result != SASL_OK)
+    {
+	VL(("Unable to get SASL_IP_LOCAL (local IP address)\n"));
+	return result;
+    }
 
     result = params->utils->getprop(params->utils->conn,
                           SASL_IP_REMOTE, (void **)&(text->ip_remote));
-    if (result != SASL_OK) return result;
+    if (result != SASL_OK)
+    {
+	VL(("Unable to get SASL_IP_REMOTE (IP address of server we're contacting)\n"));
+	return result;
+    }
 
     oparams->authid =
       params->utils->malloc(strlen(text->credentials.pname)
