@@ -37,6 +37,9 @@ __declspec(dllimport) int getsubopt(char **optionp, char * const *tokens, char *
 #endif /* WIN32 */
 #include <sasl.h>
 #include <saslutil.h>
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
 
 #ifndef HAVE_GETSUBOPT
 int getsubopt(char **optionp, char * const *tokens, char **valuep);
@@ -753,13 +756,16 @@ main(int argc, char *argv[])
 			      &data, &len);
     if (result != SASL_OK && result != SASL_CONTINUE)
       saslfail(result, "Performing SASL negotiation", NULL);
+
+    puts("Sending response...");      
     if (data) {
-      puts("Sending response...");
       samp_send(data, len);
 #ifndef WIN32
       /* This free causes win32 to fail */
       free(data);
 #endif /* WIN32 */
+    } else {
+      samp_send("",0);
     }
   }
   puts("Negotiation complete");
